#ifndef __GRAMMAR_H
#define __GRAMMAR_H

#include "Token.h"

#include <vector>
#include <map>
#include <set>

using namespace std;

// -- Context-free grammar --
class Grammar {
public:
	multimap<Token*, vector<Token*> > rules;	// Rules ("l -> u", u being a vector of token)
	Token* axiom;								// Starting token
	set<Token*> lexGram;						// Set of tokens appearing somewhere in the grammar
	
	map<Token*, set<Token*> > first;			// Computed through "fst_follow()"
	map<Token*, set<Token*> > follow;			// Computed through "fst_follow()"
	map<Token*, bool> nullable;					// Computed through "fst_follow()"
	
	
	// ** Constructor
	Grammar(multimap<Token*, vector<Token*> > rl, Token* ax);	// Grammar containing the axioms ax.
	
	// ** Destructor
	~Grammar();
	
	/*
	 * Compute the attributes nullable, first and follow where:
	 * - Nullable(u) = true iff u --*--> \epsilon
	 * - First(u) = { a \in lexGram  | u --*--> v_1.a.v_2  /\  a is terminal /\ v_1 --*--> \epsilon }
	 * - Follow(X) = { a \in lexGram | Z --*--> uXav}
	 */
	void fst_follow();
	
	
	// Compute and return the director of (A-> u) as a mapping from the rule to the set.
	// WARNING: you need to execute fst_follow() before applying this method.
	map<pair<Token*, vector<Token*> >, set<Token*> > director();
	
	// Compute the table of predictive analysis T[X][a] = rule (type of rule = pair<Token*, vector<Token*> >)
	// WARNING: the grammar must be LL(1)
	map<Token*, map<Token*, pair<Token*, vector<Token*> > > > tableAnalysePred(map<pair<Token*, vector<Token*> >, set<Token*> > direct);
	
	// TODO: reste plus qu'à implémenter la table d'analyse prédictive...
	
	
	
};

// Pretty-printer
ostream& operator <<(ostream& sout, Grammar& gram);


// Misc method: pretty-printer for structures
ostream& operator <<(ostream& sout, map<Token*, set<Token*> >& mapping);	// For first/follow

ostream& operator <<(ostream& sout, map<Token*, bool >& mapping);	// For nullable

ostream& operator <<(ostream& sout, map<pair<Token*, vector<Token*> >, set<Token*> >& mapping);	// For director

ostream& operator <<(ostream& sout, map<Token*, map<Token*, pair<Token*, vector<Token*> > > >& mapping);	// For director

#endif
